@@ -15,3 +15,8 @@ def greeting(match):
 def say(match):
     for part in match[1].split(', '):
         yield 'Saying {}', part
+
+
+@command(r'colors')
+def colors(match):
+    return '{u}{k}04C{k}05o{k}06l{k}o{k}07r{k}08{k}09s{k}!'
